library(brms)
library(tidyverse)
library(bayesplot)
library(rethinking)
library(ggplot2)
bayesplot::color_scheme_set("darkgray")

# For reproducibility.
SEED <- 60511 # My oldest daughter was born 2016, June 11.
set.seed(SEED)

#Read the data into a dataframe.
y <- read.csv("obsoletepredictions.csv", sep=";")
#Look at the data frame
glimpse(y)


#Create histograms for the raw data. We see that we of course have a bimodal distribution in our data. 
Actual <- hist(y$actual)
Estimate <- hist(y$estimate)
plot(Actual)
plot(Estimate)

Obsolete <- hist(y$obsolete)
TotalReq <- hist(y$totalreq)
plot(Obsolete)
plot(TotalReq)



#A quick summary of what we did here: We wanted to create a model that could predict the overestimations by knowing
#the percent of obsolete requirements. This was not easy. We only have data from the magnitude of the overestimations from Exp4.
#Fitting a bimodal distribution for this kind of data is hard for Stan. We ha take the logarith of b to get a good model. 

y$y <- y$estimate/y$actual
y$b <- y$obsolete/y$totalreq


y$b_log <- log(y$b)

#Our mdoel is difficult because we also added variying intercepts for the data from exp 4 and 5...
M0.3 <- brm(formula = y ~ 1 + b_log + (1|exp),
            data = y,
            prior = c(prior(normal(0, 1), class = Intercept),
                        prior(normal(0, 0.1), class = b),
                        prior(cauchy(0, 0.5), class = sigma)),
            family = gaussian,
                        seed = SEED,
            sample_prior = "yes",
            cores = 2,
            control = list(adapt_delta=0.999999999999,max_treedepth=14),
)#you might need to compil this model a few times for it to work. Seems like we're pshing the limit here a bit. 

M<-M0.3






#Rhat is also a way to check how chains mix. This value is value ok.
max(rhat(M))

#Now let's check if the predictions based on the model and the data (i.e. the posterior) fit the data.
ppc_dens_overlay(y=(y$y), 
                 yrep=posterior_predict(M, nsamples=100)) + 
  theme(legend.position="none",
        axis.text.x=element_blank(),
        axis.ticks.x=element_blank()) 
#We are now happy with the result. It underestimates a bit and overestimates to the right, but we have so few datapoints and a tricky model so 
#we'll settle now. 




#Let's check how our chians mix visually.
plot(M)
# Our different levels on "exp", whcih is from experiment 4 or 5 is what causes us trouble. But we want to push for a model that can let us 
#predict values for Exp6. 




#Looks like all the groups are (very) significantly different from each other! As a comparison, even with overlapping 
#distributions, the differences are often significant if they are not too flat, which they aren't in this case.
#We can also look at the values:
M



df <- data.frame(exp="E5", b_log=log(0.21))
post_e5_021 <- posterior_predict(M, newdata=df)
plot(density(post_e5_021))
HPDI(post_e5_021, prob = 0.95)
summary(post_e5_021)

df <- data.frame(exp="E5", b_log=log(0.1))
post_e5_01 <- posterior_predict(M, newdata=df)
plot(density(post_e5_01))
HPDI(post_e5_01, prob = 0.95)
summary(post_e5_01)


df <- data.frame(exp="E5", b_log=log(0.2))
post_e5_02 <- posterior_predict(M, newdata=df)
plot(density(post_e5_02))
HPDI(post_e5_02, prob = 0.95)
summary(post_e5_02)



df <- data.frame(exp="E5", b_log=log(0.22))
post_e5_022 <- posterior_predict(M, newdata=df)
plot(density(post_e5_022))
HPDI(post_e5_022, prob = 0.95)
summary(post_e5_022)

